local Smoother = LibStub("LibSmoothStatusBar-1.0")
local Glower = LibStub("LibCustomGlow-1.0")
local _G = getfenv(0)

function SliceCommander:NewFrame(color, name)
	local f = CreateFrame("StatusBar", 'SliceCommander_bar_' .. name, UIParent, BackdropTemplateMixin and "BackdropTemplate")
	--Mixin(f, "Region")
	Smoother:SmoothBar(f)
	
	f:SetStatusBarTexture(SliceCommander:BarTexture())
	f:SetValue(0)
	f:SetDefaultBackdrop()
	f:SetBackdropColor(unpack({.1,.1,.1,.7}))
	f:SetFrameStrata("MEDIUM")

	-- text on the right --
	f.text = f:CreateFontString(nil, nil, 'GameFontNormal')
	f.text:SetFontObject(_G['SC'].BarFont1)
	f.text:SCWidth(120)
	f.text:SCPoint("TOPRIGHT", f, "TOPRIGHT", -5, 0)
	f.text:SetJustifyH("RIGHT")

	f.icon = f:CreateTexture(nil, "OVERLAY", nil, 1)
	f.icon:SCPoint("RIGHT", f, "LEFT", -2, 0)
	f.icon:SetTexCoord(.08, .92, .08, .92)

	-- text on the center --
	f.text2 = f:CreateFontString(nil, nil, 'GameFontNormal')
	f.text2:SCPoint("CENTER", f, "CENTER", -8, 0)
	f.text2:SetFontObject(_G['SC'].BarFont1)
	f.text2:SCWidth(200)
	f.text2:SetJustifyH("CENTER")

	f:SetStatusBarColor(unpack(color))
	f.border = SliceCommander:tukSkin(f, false)
	f.iconBorder = SliceCommander:tukSkin(f, f.icon)
	
	local animPlaying = false

	--Animation (Blink)
	f.animFade = f:CreateAnimationGroup()
	f.animFade:SetLooping('BOUNCE')
	
	local animOut = f.animFade:CreateAnimation("Alpha")
	animOut:SetFromAlpha(1)
    animOut:SetToAlpha(.3)
    animOut:SetDuration(.4)
    animOut:SetSmoothing("OUT")
	animOut:SetOrder(1)

	--Animation (Glow)
	--local animFrame = CreateFrame("Frame", nil, UIParent, BackdropTemplateMixin and "BackdropTemplate")
	--animFrame:SetAllPoints(f)
	f.anim = {
		Play = function()
			animPlaying = true
			Glower.PixelGlow_Start(f)
			f.animFade:Play()
		end,
		Stop = function()
			animPlaying = false
			Glower.PixelGlow_Stop(f)
			f.animFade:Stop()
		end,
		IsPlaying = function()
			return animPlaying
		end
	}
	
	return f

end

function SliceCommander:CPFrame()
	local f = CreateFrame("StatusBar", nil, UIParent, BackdropTemplateMixin and "BackdropTemplate")
	local colorAnim = {.6, .6, 1, 1}

	f.bg = f:CreateTexture(nil, "BACKGROUND", nil, 1)
	f.bg:SCPoint("TOPLEFT", f, "TOPLEFT", 0, 0)
	f.bg:SCPoint("BOTTOMRIGHT", f, "BOTTOMRIGHT", 0, 0)
	f:SetFrameStrata("MEDIUM")

	f.combos = {}

	for i = 1, 10 do--Maximum CP is 10 with anticipation
		local combo = CreateFrame("Frame", nil, f, BackdropTemplateMixin and "BackdropTemplate")
		combo.bg = combo:CreateTexture(nil, "BACKGROUND", nil, 1)
		combo.bg:SetAllPoints(combo)
		combo.bg:SetTexture("Interface\\AddOns\\SliceCommander\\Images\\solid.tga")
		combo.border = SliceCommander:tukSkin(combo, false)
		combo.bg:Hide()
		
		local animPlaying = false
		combo.anim = {
			Play = function()
				animPlaying = true
				Glower.PixelGlow_Start(combo, colorAnim, 8, .5)
			end,
			Stop = function()
				animPlaying = false
				Glower.PixelGlow_Stop(combo)
			end,
			IsPlaying = function()
				return animPlaying
			end
		}
		f.combos[i] = combo
	end

	f.overlay = CreateFrame("Frame", nil, f, BackdropTemplateMixin and "BackdropTemplate")
	f.overlay:SetAllPoints(f)
	
	f.comboText = f.overlay:CreateFontString(nil, 'OVERLAY', 'GameFontNormal')
	f.comboText:SetFont(SliceCommander:GetFont(), 8, "THINOUTLINE")
	f.comboText:SetShadowColor(0, 0, 0, 0.4)
	f.comboText:SetTextColor(1, 1, 1)
	f.comboText:SetShadowOffset(1, -1)	
	f.comboText:SetJustifyH("CENTER")
	f.comboText:SCWidth(120)
	f.comboText:SetAllPoints(f.overlay)

	f:Hide()
	return f

end

function SliceCommander:ThinBar(point, relativePoint, x, y)
	local f = CreateFrame("StatusBar", nil, UIParent, BackdropTemplateMixin and "BackdropTemplate")
	
	Smoother:SmoothBar(f)
	
	f:SCHeight(2)
	f:SetFrameLevel(2)
	f:SetStatusBarTexture(SliceCommander:BarTexture())
	f:SetStatusBarColor(64/255, 167/255, 64/255)
	f:SetMinMaxValues(0, 3600*1000)
	f:SetFrameStrata("MEDIUM")
	f:Hide()
	f:SetDefaultBackdrop()
	local r,v,b = unpack(_G['SC'].V.bColor)
	f:SetBackdropBorderColor(unpack(_G['SC'].V.wColor))
	f:SetBackdropColor(r,v,b,0)
	f:SetPoint(point, VTimerEnergy, relativePoint, x, y)
	return f
end

function SliceCommander:CreateMoveFrame()
	CreateFrame('Frame', 'SliceCommanderFrame', UIParent, BackdropTemplateMixin and "BackdropTemplate")
	SliceCommanderFrame:SetToplevel(true)
	SliceCommanderFrame:EnableMouse(false)
	SliceCommanderFrame:SetMovable(true)
	SliceCommanderFrame:SCSize(216, 36)
	SliceCommanderFrame:SetPoint(
		_G['SC'].OTHER['SliceCommanderFrame']['position'].point,
		_G['SC'].OTHER['SliceCommanderFrame']['position'].relativeTo,
		_G['SC'].OTHER['SliceCommanderFrame']['position'].relativePoint,
		_G['SC'].OTHER['SliceCommanderFrame']['position'].xOfs,
		_G['SC'].OTHER['SliceCommanderFrame']['position'].yOfs
	)
	SliceCommanderFrame:SetDefaultBackdrop()
	SliceCommanderFrame:SetBackdropColor(0.7,0.2,0.2,1)
	SliceCommanderFrame:SetScript("OnMouseDown",function () SliceCommander:MoveStart(SliceCommanderFrame) end)
	SliceCommanderFrame:SetScript("OnMouseUp",function () SliceCommander:MoveStop(SliceCommanderFrame) end)
	SliceCommanderFrame:SetScript("OnUpdate",function () SliceCommander:OnUpdate() end)
end

function SliceCommander:CreateEnergyFrame()
	local f = CreateFrame('StatusBar', 'VTimerEnergy', UIParent, BackdropTemplateMixin and "BackdropTemplate")
	Smoother:SmoothBar(f)
	f:SCSize(200, 20)
	f:SetMinMaxValues(0,100)
	f:SCPoint('CENTER', SliceCommanderFrame, 'CENTER', 0, 0)
	f:SetFrameLevel(1)
	f:SetMinMaxValues(0,UnitPowerMax("player", Enum.PowerType.Energy))
	f:SetDefaultBackdrop()
	f:SetBackdropBorderColor(unpack(_G['SC'].V.wColor))
	f:SetBackdropColor(unpack({.1,.1,.1,.7}))
	f:SetStatusBarTexture(SliceCommander:BarTexture())
	f:SetFrameStrata("MEDIUM")
	f:SetFrameLevel(1)
	f:SetStatusBarColor(unpack(_G['SC'].SET.energyColor))
	f.border = SliceCommander:tukSkin(f, false)
	if _G['SC'].SET.HideEnergy then
		f:Hide()
	else
		f:Show()
	end

	f.txt = f:CreateFontString('VTimerEnergyTxt', 'OVERLAY', 'GameFontNormal')
	f.txt:SetJustifyH("RIGHT")
	f.txt:SCSize(49, 20)
	f.txt:SCPoint('RIGHT', VTimerEnergy, 'RIGHT', -5, 0)
	f.txt:SetFontObject(SliceCommander:NewFont("VTimerFont", 17, 0.7, 1, 1, 1))
end

function SliceCommander:CreateComboText()
	VTimerEnergy:CreateFontString('SliceCommander_Combo', 'OVERLAY', 'GameFontNormal')
	SliceCommander_Combo:SetFont(SliceCommander:GetFont(), 17, "THINOUTLINE")
	SliceCommander_Combo:SetShadowColor(0, 0, 0, 0.4)
	SliceCommander_Combo:SetTextColor(1, 1, 1)
	SliceCommander_Combo:SetShadowOffset(1, -1)	
	SliceCommander_Combo:SetJustifyH("LEFT")
	SliceCommander_Combo:SCSize(40, 20)
	SliceCommander_Combo:SCPoint('TOPLEFT', VTimerEnergy, 'TOPLEFT', 5, 0)
end

function SliceCommander:CreateSpark()
	for i = 1, 3 do
		_G['SC'].Spark[i] = VTimerEnergy:CreateTexture(nil, 'OVERLAY', nil, 1)
		_G['SC'].Spark[i]:SetTexture("Interface\\CastingBar\\UI-CastingBar-Spark")
		_G['SC'].Spark[i]:SetBlendMode('ADD')
		_G['SC'].Spark[i]:SCSize(5,20)
	end
end

function SliceCommander:NewTickArray(BAR)
	local tickObj = BAR['obj']:CreateTexture(nil, 'OVERLAY', nil, 1)
	tickObj:SetTexture('Interface\\AddOns\\SliceCommander\\Images\\solid.tga')
	tickObj:SCWidth(2)
	tickObj:SCHeight(_G['SC'].SET.TimerHeight)
	tickObj:SetVertexColor(226/255, 226/255, 226/255, 0.75)
	table.insert(_G['SCtick'], tickObj)
	return tickObj
end

function SliceCommander:NewMoveFrameIcon(name)
	local MoveFrame = CreateFrame('Button', name, UIParent, BackdropTemplateMixin and "BackdropTemplate")
	MoveFrame:SetToplevel(true)
	MoveFrame:EnableMouse(false)
	MoveFrame:SetMovable(true)
	MoveFrame:SCSize(28, 28)
	MoveFrame:SetPoint(
		_G['SC'].OTHER[name]['position'].point,
		_G['SC'].OTHER[name]['position'].relativeTo,
		_G['SC'].OTHER[name]['position'].relativePoint,
		_G['SC'].OTHER[name]['position'].xOfs,
		_G['SC'].OTHER[name]['position'].yOfs
	)
	MoveFrame:SetDefaultBackdrop()
	MoveFrame:SetBackdropColor(0.7,0.2,0.2,1)
	MoveFrame:SetScript("OnMouseDown",function () SliceCommander:MoveStart(MoveFrame) end)
	MoveFrame:SetScript("OnMouseUp",function () SliceCommander:MoveStop(MoveFrame) end)

	return MoveFrame
end

function SliceCommander:NewIcon(icon, name)
	local MoveFrame = SliceCommander:NewMoveFrameIcon(name)

	local f = CreateFrame("Frame", name..'button', UIParent, BackdropTemplateMixin and "BackdropTemplate")
	f:SCPoint('CENTER', name, 'CENTER', 0, 0)
	f:SetFrameStrata("MEDIUM")
	f:SetFrameLevel(1)
	f.moveframe = MoveFrame

	local fIcon = f:CreateTexture(nil, "OVERLAY", nil, 1)
	fIcon:SCPoint("CENTER", f, "CENTER", 0, 0)
	fIcon:SetAlpha(1)
	fIcon:SetTexCoord(.08, .92, .08, .92)
	fIcon:SetTexture(icon)
	f.icon = fIcon
	f.border = SliceCommander:tukSkin(f, fIcon)

	f.overlay = CreateFrame('Frame', nil, f, BackdropTemplateMixin and "BackdropTemplate")
	f.overlay:SetFrameStrata("HIGH")
	f.overlay:SetFrameLevel(2)
	f.overlay:SCPoint('CENTER', f:GetName(), 'CENTER', 0, 0)
	f.overlay:SetDefaultBackdrop()
	f.overlay:SetBackdropColor(0,0,0,.3)
	f.overlay:Hide()

	f.iconTxt = f.overlay:CreateFontString(nil, 'OVERLAY', 'GameFontNormal')
	f.iconTxt:SetJustifyH("CENTER")
	f.iconTxt:SetJustifyV("MIDDLE")
	f.iconTxt:SCPoint('CENTER', f, "CENTER", 2, 0)
	f.iconTxt:SetFontObject(_G['SC'].FontIcon)
	f.iconTxt:SetShadowColor(0,0,0,1)
	f.iconTxt:SetShadowOffset(2*_G['SC'].V.pxW, -2*_G['SC'].V.pxH)

	return f
end

function SliceCommander:NewFont(name, FontSize, aShadow, rColor, gColor, bColor)
	local f = CreateFont(name)
	f:SetFont(SliceCommander:GetFont(), FontSize, "")
	f:SetShadowColor(0, 0, 0, aShadow)
	f:SetTextColor(rColor, gColor, bColor, 1)
	f:SetShadowOffset(0.8, -0.8)

	return f
end

function SliceCommander:CreateTextureSkin(frame, color, level, texture)
	local t = CreateFrame('Frame', nil, frame, BackdropTemplateMixin and "BackdropTemplate")
	t:SetDefaultBackdrop()
	t:SetFrameLevel(level)
	t:SetFrameStrata("LOW")
	t:SetBackdropColor(unpack(color))
	if texture then
		t:SCPoint("CENTER", texture, "CENTER", 0, 0)
	else
		t:SCPoint("CENTER", frame, "CENTER", 0, 0)
	end
	t:Show()

	return t
end

function SliceCommander:tukSkin(frame, texture)
	return {
		[1] = SliceCommander:CreateTextureSkin(frame, _G['SC'].V.bColor, 1, texture),
		[2] = SliceCommander:CreateTextureSkin(frame, _G['SC'].V.wColor, 2, texture),
		[3] = SliceCommander:CreateTextureSkin(frame, _G['SC'].V.bColor, 3, texture)
	}
end